@@ -54,6 +54,7 @@ read_oo_pidata <- function(file,
     col_names = c("w.length", "counts"),
     skip = 5,
     n_max = npixels,
+    col_types = readr::cols(),
     locale = locale
   )
   
@@ -68,6 +69,7 @@ read_oo_pidata <- function(file,
   photobiology::setWhenMeasured(z, date)
   photobiology::setWhereMeasured(z, geocode)
   photobiology::setWhatMeasured(z, label)
+  attr(z, "file.header", file_header)
   z
 }
 
